$CASE
$DEBUG
; %'@(#)cstart.asm      1.5     6/29/90'
;       Header file for 8051 type processors for C8051
;       -----------------------------------------------
;
;       When using the reentrant model, and interrupt protection of updating
;       the virtual stackpointer is required. Define PROTECT as YES
;
        NAME    _CSTART 
;
        EXTRN   CODE(_main)     ; start label user program
        PUBLIC  __STKSTART      ; start of stack (also used by XRAY51)
        PUBLIC  __HEAPSTART     ; start of heap area
        PUBLIC  __HEAPLENGTH    ; end of heap area
        PUBLIC  __EXIT          ; address to jump to on 'exit()'

%DEFINE(HEAP)           (800H)          ; define a heap (room for malloc in xdat) of 2 Kbytes
%DEFINE(STACKLENGTH)    (20H)           ; define a stacklength of 32 bytes
%DEFINE(RAMSIZE)        (03FH)          ; size of IDAT memory (3f for 8xc751)
%DEFINE(XDATSTART)      (0H)            ; start address of XDAT
%DEFINE(XDATEND)        (2000H)         ; end address of XDAT + 1 (size is 8 kBytes)
%DEFINE(REGBANK)        (0)             ; default register bank

%DEFINE(PROTECT)        (NO)            ; default no protection is done on updating the
                                        ; virtual stack pointer (reentrant model only)
;
; the following code is needed for initialization of static variables
;
;       MODEL can have values (SMALL), (AUX), (LARGE) or (REENTRANT)
;       it defines in which area the strings are located.
;
  %DEFINE(MODEL)(SMALL)
;       next file will define this variable

%IF(%EQS(%MODEL,REENTRANT)) THEN (
%DEFINE(VIRT_STACK)     (400H)          ; define a virtual stack in xdat of 1Kbyte
) FI

;
; Definition of some names used by XRAY
;
        DSEG AT 0
__simulated_output      DATA    $+99H   ; Used by XRAY, output to SBUF will be done to  standard I/O device.
__simulated_input       DATA    $+99H   ; Used by XRAY, input from SBUF will be done from standard I/O device.

CINIT_ROM_XD    SEGMENT CODE ROMDATA                    ; define labels needed for initialization
        RSEG    CINIT_ROM_XD
__START_ROM_XD:
CINIT_ROM_ID    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_ID
__START_ROM_ID:
CINIT_ROM_PD    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_PD
__START_ROM_PD:
CINIT_ROM_BA    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_BA
__START_ROM_BA:
CINIT_ROM_BI    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_BI
__START_ROM_BI:
CINIT_ROM_DA    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_DA
__START_ROM_DA:
CINIT_ROM_ST    SEGMENT CODE ROMDATA
        RSEG    CINIT_ROM_ST
__START_ROM_ST:

CINIT_RAM_XD    SEGMENT XDATA
        RSEG    CINIT_RAM_XD
__START_RAM_XD:
CINIT_RAM_ID    SEGMENT IDATA
        RSEG    CINIT_RAM_ID
__START_RAM_ID:
CINIT_RAM_PD    SEGMENT XDATA
        RSEG    CINIT_RAM_PD
__START_RAM_PD:
CINIT_RAM_BA    SEGMENT DATA    BITADDRESSABLE
        RSEG    CINIT_RAM_BA
__START_RAM_BA:
CINIT_RAM_BI    SEGMENT BIT
        RSEG    CINIT_RAM_BI
__START_RAM_BI:
CINIT_RAM_DA    SEGMENT DATA
        RSEG    CINIT_RAM_DA
__START_RAM_DA:
%IF(%EQS(%MODEL,SMALL)) THEN (
CINIT_RAM_ST    SEGMENT DATA                    ; This is the small memory model, so strings are allocated in data
)FI
%IF(%EQS(%MODEL,AUX)) THEN (
CINIT_RAM_ST    SEGMENT XDATA                   ; This is the auxiliary memory model, so strings are allocated in pdata
)FI
%IF(%EQS(%MODEL,LARGE)) THEN (
CINIT_RAM_ST    SEGMENT XDATA                   ; This is the large memory model, so strings are allocated in xdata
)FI
%IF(%EQS(%MODEL,REENTRANT)) THEN (
CINIT_RAM_ST    SEGMENT XDATA                   ; This is the reentrant memory model, so strings are allocated in xdata
)FI
        RSEG    CINIT_RAM_ST
__START_RAM_ST:

        EXTRN   CODE    (__END_ROM_XD)                  ; end labels needed for initialization
        EXTRN   CODE    (__END_ROM_ID)
        EXTRN   CODE    (__END_ROM_PD)
        EXTRN   CODE    (__END_ROM_BA)
        EXTRN   CODE    (__END_ROM_BI)
        EXTRN   CODE    (__END_ROM_DA)
        EXTRN   CODE    (__END_ROM_ST)

        EXTRN   CODE    (__INITBIT)                     ; routines used from library
        EXTRN   CODE    (__INITCID)
        EXTRN   CODE    (__INITCDA)
        EXTRN   CODE    (__INITCPD)
        EXTRN   CODE    (__INITCXD)
        EXTRN   CODE    (__INITB)

;
?STACK  SEGMENT IDATA
        RSEG    ?STACK
__STKSTLAB:
        DS      %STACKLENGTH            ; reserve space for the stack
__STKSTART      IDATA   __STKSTLAB - 1
;
?HEAP   SEGMENT XDATA
        RSEG    ?HEAP
__HEAPSTART:
        DS      %HEAP           ; reserve space for the heap
__HEAPEND:
__HEAPLENGTH    XDATA   __HEAPEND - __HEAPSTART ; length of the heap

%IF(%EQS(%MODEL,REENTRANT)) THEN (
PUBLIC  __TOP_OF_VIRT_STACK
?VIRT_STACK     SEGMENT XDATA
        RSEG    ?VIRT_STACK
        DS      %VIRT_STACK     ; reserve space for the virtual stack in external RAM
__TOP_OF_VIRT_STACK:
) FI
;
        CSEG    AT 0H
        AJMP    __START
;
;       Call the user program
;

        CSEG AT 2CH              ;was 05BH
__START:
        ; First we have to do some initialisation. Disable the interrupts (because an 'exit' will jump to this point also)
        ; and clear the memory. Also make sure which register bank is the current used one.
        CLR     EA                              ; disable interrupts
        CLR     A
        MOV     PSW,    A                       ; select register bank 0
        ; First initialize all IDATA to '0'
        MOV     R0,   #08                       ; one position after registerbank 0
__CLR_IDAT:
        MOV     @R0,  A
        INC     R0
        CJNE    R0,   #%RAMSIZE, __CLR_IDAT     ; RAMSIZE must be 03F for 8xC751

        MOV     SP,     #LOW(__STKSTART)        ; initialise stack pointer
;        MOV     P2,     #00                     ; needed for models using pdat

        MOV     DPTR,   #__START_ROM_ID         ; Initialize IDAT
        MOV     R0,     #__START_RAM_ID
        MOV     R6,     #HIGH(__END_ROM_ID)
        MOV     R7,     #LOW(__END_ROM_ID)
        ACALL   __INITCID

        MOV     DPTR,   #__START_ROM_DA         ; Initialize DATA
        MOV     R0,     #__START_RAM_DA
        MOV     R6,     #HIGH(__END_ROM_DA)
        MOV     R7,     #LOW(__END_ROM_DA)
        ACALL   __INITCDA

        MOV     DPTR,   #__START_ROM_BA         ; Initialize BIT ADDRESSABLE area (in RAM)
        MOV     R0,     #__START_RAM_BA
        MOV     R6,     #HIGH(__END_ROM_BA)
        MOV     R7,     #LOW(__END_ROM_BA)
        ACALL   __INITBIT

%IF(%EQS(%MODEL,SMALL)) THEN (
        ; Small model (-Ms), strings are in DATA
        EXTRN   CODE    (__INITSTRING_DA)
        MOV     DPTR,   #__START_ROM_ST         ; Initialize STRING area in DATA
        MOV     R0,     #__START_RAM_ST
        MOV     R6,     #HIGH(__END_ROM_ST)
        MOV     R7,     #LOW(__END_ROM_ST)
        ACALL   __INITSTRING_DA
)FI
%IF(%EQS(%MODEL,AUX)) THEN (
        EXTRN   CODE    (__INITSTRING_PD)
        ; auxiliary model (-Ma), strings are in PDAT
        MOV     DPTR,   #__START_ROM_ST         ; Initialize STRING area in PDATA
        MOV     R4,    #HIGH( __START_RAM_ST)
        MOV     R5,    #LOW(__START_RAM_ST)
        MOV     R6,    #HIGH( __END_ROM_ST)
        MOV     R7,    #LOW(__END_ROM_ST)
        ACALL   __INITSTRING_PD
)FI
%IF(%EQS(%MODEL,LARGE)) THEN (
        EXTRN   CODE    (__INITSTRING_XD)
        MOV     DPTR,   #__START_ROM_ST         ; Initialize STRING area in XDATA
        MOV     R4,    #HIGH( __START_RAM_ST)
        MOV     R5,    #LOW(__START_RAM_ST)
        MOV     R6,    #HIGH( __END_ROM_ST)
        MOV     R7,    #LOW(__END_ROM_ST)
        ACALL   __INITSTRING_XD
        ; Large model (-Ml), strings are in XDAT
)FI
%IF( %NES(%PROTECT,YES) AND %EQS(%MODEL,REENTRANT) ) THEN (
        EXTRN   CODE    (__INITSTRING_XD)
        MOV     DPTR,   #__START_ROM_ST         ; Initialize STRING area in XDATA
        MOV     R4,    #HIGH( __START_RAM_ST)
        MOV     R5,    #LOW(__START_RAM_ST)
        MOV     R6,    #HIGH( __END_ROM_ST)
        MOV     R7,    #LOW(__END_ROM_ST)
        ACALL   __INITSTRING_XD
        ; Reentrant model (-Mr), strings are in XDAT
        EXTRN   DATA( __SP,__BP )
        MOV     __SP,   #HIGH( __TOP_OF_VIRT_STACK )
        MOV     __SP+1, #LOW( __TOP_OF_VIRT_STACK )
        MOV     __BP,__SP
        MOV     __BP+1,__SP+1
) FI
%IF( %EQS(%PROTECT,YES) AND %EQS(%MODEL,REENTRANT) ) THEN (
        EXTRN   CODE    (__INITSTRING_XD)
        MOV     DPTR,   #__START_ROM_ST         ; Initialize STRING area in XDATA
        MOV     R4,    #HIGH( __START_RAM_ST)
        MOV     R5,    #LOW(__START_RAM_ST)
        MOV     R6,    #HIGH( __END_ROM_ST)
        MOV     R7,    #LOW(__END_ROM_ST)
        ACALL   __INITSTRING_XD
        ; Reentrant model (-Mr), strings are in XDAT
        EXTRN   DATA( __SP,__BP )
        PUSH    0A8H                            ; save interrupt flag (IE)
        CLR     0AFH                            ; No interrupts allowed
        MOV     __SP,   #HIGH( __TOP_OF_VIRT_STACK )
        MOV     __SP+1, #LOW( __TOP_OF_VIRT_STACK )
        POP     0A8H                            ; restore interrupt flag
        MOV     __BP,__SP
        MOV     __BP+1,__SP+1
) FI

; 8xC751 has no PDAT or XDAT
;
;        MOV     DPTR,  #__START_ROM_PD
;        MOV     R4,    #HIGH( __START_RAM_PD)   ; Initialize PDAT
;        MOV     R5,    #LOW(__START_RAM_PD)
;        MOV     R6,    #HIGH( __END_ROM_PD)
;        MOV     R7,    #LOW(__END_ROM_PD)
;        ACALL   __INITCPD
;
;        MOV     DPTR,  #__START_ROM_XD
;        MOV     R4,    #HIGH( __START_RAM_XD)   ; Initialize XDAT
;        MOV     R5,    #LOW(__START_RAM_XD)
;        MOV     R6,    #HIGH( __END_ROM_XD)
;        MOV     R7,    #LOW(__END_ROM_XD)
;        ACALL   __INITCXD
;
        MOV     DPTR,   #__START_ROM_BI         ; Initialise BIT area
        MOV     R0,     #__START_RAM_BI
        MOV     R6,     #HIGH(__END_ROM_BI)
        MOV     R7,     #LOW(__END_ROM_BI)
        ACALL   __INITB

        MOV     PSW,    #(%REGBANK * 8)         ; select correct registerbank

        USING   %REGBANK
        ACALL   _main                           ; Jump to C-program
__EXIT:
__STOP:
        DB      0A5H                            ; illegal instruction halts simulator
        SJMP    __STOP

END
