$TITLE(I2C age Handler)
;*==========================================================*
;*                                                          *
;*         SOURCE FILE : I2C_HAND.ASM                       *
;*         PACKAGE     : I2C                                *
;*                                                          *
;*==========================================================*
$DEBUG


;*==========================================================*
;*    I N C L U D E S                                       *
;*==========================================================*
$NOLIST
$INCLUDE(REG751.H)
$INCLUDE(I2C_DATA.GLO)
$INCLUDE(I2C_DATA.LOC)
$LIST


;*==========================================================*
;*    G L O B A L   R E F E R E N C E S                     *
;*==========================================================*
    EXTRN CODE(I2C_STOP)
    EXTRN CODE(I2C_TRX_BYTE)
    EXTRN CODE(I2C_TRX_ADDR)
    EXTRN CODE(I2C_RCV_BYTE)
    EXTRN CODE(I2C_TRX_BLOCK)
    EXTRN CODE(I2C_RCV_BLOCK)
    EXTRN CODE(I2C_STRT_SLVAD)
    EXTRN CODE(I2C_RSTRT_SLVAD)

;*==========================================================*
;*    G L O B A L   F U N C T I O N   D E F I N I T I O N S *
;*==========================================================*
    PUBLIC I2C_MESS_HAND


;*==========================================================*
;*    L O C A L   S Y M B O L   D E C L A R A T I O N S     *
;*==========================================================*
    RWN         BIT     0E0H        ;bit ACC.0
    I2C_PSW     EQU     8


;*==========================================================*
;*    C O D E   S E G M E N T                               *
;*==========================================================*
    I2C_DRIVER SEGMENT CODE
    RSEG I2C_DRIVER


;*MPF:::I2C::I2C_HAND.ASM:I2C_MESS_HAND=====================*
;*                                                          *
;* FUNCTION NAME:      I2C_HAND                             *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*   Transmit an I2C age, includes error handling       *
;*                                                          *
;* INPUT: age control byte I2C_CTRL (bit addressable)   *
;*        age control block I2C_MCB, containing:        *
;*          I2C_ADDR1       (i.e. slave address)            *
;*          BUF_LEN1        (i.e. number of bytes to trx.)  *
;*          BUF_PTR1        (i.e. transmit buffer)          *
;*          I2C_ADDR2       (i.e. sub address)              *
;*          BUF_LEN2        (i.e. lenght of second data blk)*
;*          BUF_PTR2        (i.e. second transmit buffer)   *
;*                                                          *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*
I2C_MESS_HAND:
        PUSH    PSW
        MOV     PSW,#I2C_PSW        ;sel RB1
        ANL     I2C_STAT,#07H       ;clr all but retry bits
        MOV     MESS_RETRY_CNT,I2C_STAT
        INC     MESS_RETRY_CNT          ;load retry counter
RETRY:
        ANL     I2C_STAT,#07H       ;clr all but retry bits
        MOV     A,I2C_ADDR_1        ;load SLV_ADDR
        CLR     RWN                 ;if (subaddress)
        JB      ADDR2_SUB,STRT      ;    RWN = 0
        MOV     C,RWN_BLK1          ;else
        MOV     RWN,C               ;    RWN = RWN_BLK1
STRT:
        ACALL   I2C_STRT_SLVAD      ;send START+SLV_ADDR+RWN
        JNB     I2C_ERR,CONTINUE    ;branch offset to large
        AJMP    EXIT
CONTINUE:
        JB      TEST_DEVICE,M_STOP
        MOV     BUF_PTR,BUF_PTR_1   ;load pointer block1
        MOV     BUF_LEN,BUF_LEN_1   ;load length block1
        JNB     ADDR2_SUB,BLOCK     ;if (addr2_sub)
        MOV     A,I2C_ADDR_2        ;  load sub address
        ACALL   I2C_TRX_BYTE        ;  trx_byte(sub address)
        JB      I2C_ERR,EXIT        ;  if (error) exit();
        JNB     REP_STRT_BLK1,BLOCK ;  if (rep. start blk1)
        MOV     A,I2C_ADDR_1        ;    load slave address
        SETB    RWN                 ;    read
        ACALL   I2C_RSTRT_SLVAD     ;    send RSTART+SLV_ADDR
        JB      I2C_ERR,EXIT        ;    if (error) exit();
BLOCK:
        JNB     RWN_BLK1,TRX_1      ;if ((rwn_blk1) == read))
        ACALL   I2C_RCV_BLOCK       ;  rcv_block(&_data1,cnt1)
        SJMP    END_BLOCK1
TRX_1:                              ;else
        ACALL   I2C_TRX_BLOCK       ;  trx_block(&data1,cnt1)
END_BLOCK1:
        JB      I2C_ERR,EXIT        ;if (error) exit();
        JNB     BLOCK2,M_STOP       ;if (2nd block of data)
        MOV     BUF_PTR,BUF_PTR_2   ;{
        MOV     BUF_LEN,BUF_LEN_2   ;
        JNB     ADDR2,DATA2         ;  if (addr2)
        JNB     REP_STRT_BLK2,DATA2 ;    if (rep. start blk2)
        MOV     A,I2C_ADDR_2        ;    { set address2
        JNB     ADDR2_SUB,SET_RWN   ;      if(addr2_sub)
        MOV     A,I2C_ADDR_1        ;        set address1 
SET_RWN:                            ;        /* same slave */
        MOV     C,RWN_BLK2
        MOV     RWN,C               ;        modify RWN_BLK1
        ACALL   I2C_RSTRT_SLVAD     ;      snd RSTART+SLV_ADDR
        JB      I2C_ERR,EXIT        ;      if (error) exit();
DATA2:                              ;    }
        JNB     RWN_BLK2,TRX_2      ;  if((rwn_blk2)==read)
        ACALL   I2C_RCV_BLOCK       ;    rcv_block(&_dat1,c1)
        SJMP    BLOCK_ERR
TRX_2:                              ;  else
        ACALL   I2C_TRX_BLOCK       ;    trx_block(&data1,c1)
BLOCK_ERR:
        JB      I2C_ERR,EXIT        ;    if (error) exit();
M_STOP:
        ACALL   I2C_STOP            ;}
        JB      I2C_ERR,EXIT        ;    if (error) exit();
        AJMP    RESTORE_CONTEXT


;*MPF:::I2C::I2C_HAND.ASM:EXIT==============================*
;*                                                          *
;* FUNCTION NAME:      EXIT                                 *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*   Exit an I2C message. This routine is only entered if an*
;*   IC error has occured. If more retries must be made,   *
;*   the message is started again. If no retry must be made,*
;*   the message handler is lesft after setting the carry.  *
;*   Carry is 1 indicates that an error has occured (return *
;*   value for C and PL/M calls). If the routine is entered *
;*   at the RESTORE_CONTEXT label, no error has occured     *
;*                                                          *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*
EXIT:
        JNB     TIME_ERR,TO_RETRY
        JNB     BUS_RECOVERED,RESTORE_CONTEXT
TO_RETRY:
        MOV     I2CON,#I2C_RELEASE
        DJNZ    MESS_RETRY_CNT,RETRY ; if (no more retries)

RESTORE_CONTEXT:
        MOV     I2CFG,#I2C_ENABLE   ; SLVEN=1,MSTRQ=0,TIRN=0
        MOV     A,I2C_STAT
        POP     PSW                 ;   restore PSW
        MOV     C,I2C_ERR
END_MESSAGE:                        ;label for debugging
        SETB    EI2                 ;   enable I2C interrupt
        RET                         ; with XRAY


;*==========================================================*
;*     H I S T O R Y                                        *
;*==========================================================*
;*                                                          *
;* 12-06-91    J.C. Pijnenburg  original version            *
;*                                                          *
;*==========================================================*
        END
