/******************************************************************
  Program do obsugi funkcji interfejsu sprztowego I2C PCF8584
  w trybie MASTER. Na podstawie AN95068 firmy Philips

  Raisonance RC-51
 ******************************************************************/

#include "i2cexprt.h"
#include "i2cdrivr.h"
#include "reg8584.h"

extern void (*masterProc)();				//obsuga transmisji w trybie master
extern void (*slaveProc)();				//obsuga transmisji w trybie slave

static I2C_TRANSFER *tfr;					//wskanik do aktywnego bufora danych
static I2C_MESSAGE *msg;					//wskanik do aktywnego bufora komunikatu

static void (*readyProc)(unsigned char, unsigned char);	//procedura wywoywana po zakoczeniu transmisji
static unsigned char mssgCount;			//liczba wysanych komunikatw
static unsigned char dataCount;			//liczba bajtw biecego komunikatu
static unsigned char state;				//stan drivera I2C


//wysanie warunku stop dla magistrali I2C, jako parametr funkcji podawany jest status drivera
static void GenerateStop(unsigned char status)
{
	CR_8584 = PIN_MASK | ESO_MASK | intMask | STO_MASK | ACK_MASK;
	master = false;
	state = ST_IDLE;
	
	readyProc(status, mssgCount);			//sygnalizacja zakoczenia operacji
}


//obsuga trybu master PCF8584
static void HandleMasterState(void)
{
	if (CS_8584 & LAB_MASK)					//master przegra w procedurze arbitrau
	{
		slaveProc();							//przejcie do trybu slave i powrt do wywoania
		GenerateStop(I2C_ARBITRATION_LOST);
		return;
	}
	
	switch (state)
	{
		case ST_SENDING:						//przesyanie bajtw
			if (CS_8584 & LRB_MASK) 
				GenerateStop(I2C_NACK_ON_DATA);
			else
				if (dataCount < msg->nrBytes) 
					DR_8584 = msg->buf[dataCount++];
				else
					if (mssgCount < tfr->nrMessages)
					{
						dataCount = 0;
						msg = tfr->p_message[mssgCount++];
						state = (msg->address & 1) ? ST_AWAIT_ACK : ST_SENDING;
						CS_8584 = ESO_MASK | STA_MASK | ACK_MASK | intMask;
						DR_8584 = msg->address;
					}
					else
						GenerateStop(I2C_OK);
			break;

		case ST_AWAIT_ACK:					//oczekiwanie na potwierdzenie od ukadu slave
			if (CS_8584 & LRB_MASK)
				GenerateStop(I2C_NACK_ON_ADDRESS);
			else
			{
				unsigned char dummy;
				
				if (msg->nrBytes == 1)
				{
					CS_8584 = ESO_MASK | intMask;
					state = ST_RECV_LAST;
				}
				else
					state = ST_RECEIVING;
				dummy = DR_8584;
			}
			break;

		case ST_RECEIVING:					//odbieranie bajtw
			if (dataCount + 2 == msg->nrBytes)
			{
				CS_8584 = ESO_MASK | intMask;
				state = ST_RECV_LAST;
			}
			msg->buf[dataCount] = DR_8584;
			break;
		case ST_RECV_LAST:
			if (mssgCount < tfr->nrMessages)
			{
				msg->buf[dataCount] = DR_8584;
				dataCount = 0;
				msg = tfr->p_message[mssgCount++];
				state = (msg->address & 1) ? ST_AWAIT_ACK : ST_SENDING;
				CS_8584 = ESO_MASK | STA_MASK | ACK_MASK | intMask;
				DR_8584 = msg->address;
			}
			else
			{
				GenerateStop(I2C_OK);
				msg->buf[dataCount] = DR_8584;
			}
			break;

		default:								//ten warunek nigdy nie powinien by wykonywany!
			GenerateStop(I2C_ERR);		//wprowadzony zosta na wszelki wypadek
			break;
	}
}


/* przeprowadza transmisj w trybie master, aplikacja wywoujca funkcj musi pozostawi blok parametrw
   nienaruszony do momentu wywoania funkcji readyProc
   wejcie: p - adres bloku parametrw, proc - adres procedury wywoywanej po zakoczeniu operacji ze
   statusem drivera przekazywanym jako parametr */
void I2C_Transfer(I2C_TRANSFER *p, void (*proc)(unsigned char, unsigned char))
{
	tfr = p;
	readyProc = proc;
	mssgCount = 0;
	dataCount = 0;
	master = true;
	msg = tfr->p_message[mssgCount++];
	
	state = (msg->address & 1) ? ST_AWAIT_ACK : ST_SENDING;
	CS_8584 = ESO_MASK | STA_MASK | ACK_MASK | intMask;
	DR_8584 = msg->address;
}


/* inicjalizacja trybu master; wejcie:warto rejestru S2 PCF8584 ustalajca szybko pracy I2C */
void InitializeMaster(unsigned char speed)
{
	state = ST_IDLE;
	readyProc = null;
	masterProc = HandleMasterState;
	AR_8584 = 0x26;
	CR_8584 = 0x20;
	CL_8584 = speed;
	CR_8584 = ESO_MASK;
	intMask = 0;
	master = false;
}
