/******************************************************************
  Program do obsugi funkcji interfejsu sprztowego I2C PCF8584
  w trybie SLAVE. Na podstawie AN95068 firmy Philips

  Raisonance RC-51
 ******************************************************************/

#include "i2cexprt.h"
#include "i2cdrivr.h"
#include "reg8584.h"

extern void (*slaveProc) ();				//funkcja obsugujca tryb slave
static unsigned char count; 				//licznik bajtw biecego komunikatu
static unsigned char slaveStatus; 		//status urzdzenia slave
static unsigned char size;					//rozmiar bufora w trybie slave
static unsigned char *slaveBuf; 			//wskanik do danych odebranych / wysyanych w trybie slave


/* funkcja obsugujca tryb slave */
void HandleSlaveState(void)
{
	switch (slaveStatus)
	{
		case SLAVE_IDLE:
			if (CS_8584 & AAS_MASK) 		//PCF8584 zaadresowany jako slave?
			{
				count = 0;
				if (DR_8584 & 1)
				{
					slaveStatus = SLAVE_TRX;		//status slave jako nadawca danych
					DR_8584 = slaveBuf[count++];	//wysanie pierwszego bajtu
				}
				else 								//slave jako odbiorca danych
					if (size> 1)
					{
						slaveStatus = SLAVE_RCV;
						CS_8584 = PIN_MASK | ESO_MASK | ACK_MASK | intMask;
					}
					else
					{
						slaveStatus = SLAVE_LAST;
						CS_8584 = PIN_MASK | ESO_MASK | intMask;
					}
				}
			break;
			
		case SLAVE_TRX:
			if (CS_8584 & LRB_MASK)
			{
				CS_8584 = PIN_MASK;				// brak ACK od urzdzenia master
				slaveStatus = SLAVE_READY; 	//ostatni przesyany bajt
			}
			else
				DR_8584 = slaveBuf[count++];	//wysanie nastpnego bajtu
			break;

		case SLAVE_RCV:
			slaveBuf[count++] = DR_8584;
			if (count == size)
			{
				CS_8584 = ESO_MASK | intMask;	//kasowanie ACK
				slaveStatus = SLAVE_LAST;
			}
			else
				CS_8584 = ESO_MASK | ACK_MASK | intMask;	//ustawienie ACK
			break;

		case SLAVE_LAST:
			slaveBuf[count] = DR_8584;
			CS_8584 = ESO_MASK | ACK_MASK | intMask;		//ustawienie ACK
			slaveStatus = SLAVE_READY; 		//odebrano ostatni bajt
			break;

		default:
			CS_8584 = PIN_MASK; 					//kasowanie przerwania
		break;
	}
}



/* wczenie trybu slave PCF8584, ustawienie prdkoci magistrali I2C
   wejcie: adres slave,, wskanik do bufora danych, rozmiar bufora, warto rejstru S2 PCF8584 */
void I2C_InitializeSlave(unsigned char slv, unsigned char *buf, unsigned char len, unsigned char speed)
{
	slaveProc = HandleSlaveState;				//funkcja obsugi drivera w trybie slave
	CR_8584 = 0;									//wyczenie funkcji PCF8584
	AR_8584 = slv; 								//ustawienie adresu slave
	CR_8584 = 0x20; 								//zapis rejestru zegara
	CL_8584 = speed;
	intMask = 0;
	size = len;
	slaveBuf = buf;
	slaveStatus = SLAVE_IDLE;
	CR_8584 = ESO_MASK;							//zaczenie interfejsu I2C
}


/* funkcja wywoywana w celu sprawdzenia stanu urzdzenia slave; uytkownik powinien zaadaptowa j do
   wasnych potrzeb (aby podejmowaa ona waciw akcj, odpowiedni dla danego statusu */
void I2C_ProcessSlave(void)
{
	switch (slaveStatus)
	{
		case SLAVE_IDLE:							//nie rb nic lub wypenij bufor do transmisji
			break;

		case SLAVE_TRX:
		case SLAVE_RCV:
		case SLAVE_LAST:							//jeli obsuga przez przerwania nie rb nic lub testuj bit PIN
			if (intMask == 0)						// 0 -> odpytywanie
			{
				if (! (CR_8584 & PIN_MASK))	//czekaj do momentu a bit PIN przyjmie warto 0
					MainStateHandler() ;
			}
			break;
		
		case SLAVE_READY :						//czytaj lub wypenij bufor dla nastpnej transmisji,
			slaveStatus = SLAVE_IDLE;			//syganlizuj stan aplikacji
			break;
	}
}
 
