$TITLE(83C751 w trybie "single master")
$MOD751


CTVAL		EQU		02h			;wartoci bitw CT1, CT0
;maski bitw rejestru I2CFG
BTIR		EQU		10h			;maska dla bitu TIRUN
BMRQ		EQU		40h			;maska dla bitu MASTRQ
;maski bitw rejestru I2CON
BCXA		EQU		80h			;CXA
BIDLE		EQU		40h			;IDLE
BCDR		EQU		20h			;CDR
BCARL		EQU		10h			;CARL
BCSTR		EQU		08h			;CSTR
BCSTP		EQU		04h			;CSTP
BXSTR		EQU		02h			;XSTR
BXSTP		EQU		01h			;XSTP
;zmienne
BitCnt	DATA		21h			;licznik bitw
ByteCnt	DATA		22h
SlvAdr	DATA		23h			;adres aktywnego ukadu slave
SubAdr	DATA		24h

RcvDat	DATA		25h			;bufor odebranych danych I2C (4 bajty, 
;od 25H do 28H)                                       
XmtDat	DATA		29h			;bufor wysyanych danych (4 bajty, od 
;29 do 2CH
StackSave	DATA	2Dh			;przechowuje adres stosu do odtworzenia
;stanu magistrali I2C
Flags		DATA		20h			;bity statusu programu
NoAck		BIT		Flags.0		;brak potwierdzenia
Fault		BIT		Flags.1		;bd transmisji
Retry		BIT		Flags.2		;wskazuje, e transmisja powinna by
;powtrzona

SCL		BIT		P0.0			;linia SCL
SDA		BIT		P0.1			;linia SDA



;**********************************************************************
; Program gwny
;********************************************************************** 

; wektory przerwa
ORG	0H
			AJMP		Reset					;wektor przerwania po reset


ORG	1Bh			;przerwanie Timera 1
TimerI:	SETB		CLRTI					;kasowanie bitu przerwania
			CLR		TIRUN
			ACALL		ClrInt				;wyczenie przerwa
			MOV		SP,StackSave		;odtworzenie stosu 
												;i powrt do progr.gwnego
			AJMP		Recover				;odtworzenie magistrali I2C
ClrInt:	RETI


;Wysanie bajtu do urzdzenia slave
;adres slave w SlvAdr, dane w XmtDat, liczba bajtw do wysania w ByteCnt.
SendData:
			CLR		NoAck					;kasowanie bitw bdw
			CLR		Fault
			CLR		Retry
			MOV		StackSave,SP		;zapamitanie adresu stosu
			MOV		A,SlvAdr				;pobranie adresu slave
			ACALL		SendAddr				;pobierz stan magistrali i 
												;wylij adres slave
			JB			NoAck,SDEX			;sprawd ack
			JB			Fault,SdatErr		;sprawd bdy transmisji
			MOV		R0,#XmtDat			;ustaw adres pocztku bufora tr.

SDLoop:	MOV		A,@R0					;pobierz dane dla slave
			INC		R0
			ACALL		XmitByte				;wylij dane do slave
			JB			NoAck,SDEX			;sprawd ack
			JB			Fault,SdatErr		;sprawd bdy transmisji
			DJNZ		ByteCnt,SDLoop

SDEX:		ACALL		SendStop				;wylij "I2C stop"

			RET

;obsuga bdu transmisji
SDatErr:	AJMP	Recover					;odtwrz stan SDA I SCL

;Odbieranie danych od urzdzenia slave
;adres slave w SlvAdr, liczba bajtw w ByteCnt, dane zwracane w RcvDat
RcvData:	CLR		NoAck					;kasowanie wskanikw bdu
			CLR		Fault
			CLR		Retry
			MOV		StackSave,SP		;zapamitanie adresu stosu
			MOV		A,SlvAdr				;pobierz adres slave
			SETB		ACC.0					;ustaw bit odczytu w adresie slave
			ACALL		SendAddr				;wylij adres slave
			JB			NoAck,RDEX			;sprawd ack
			JB			Fault,RdatErr		;sprawd bdy transmisji

			MOV		R0,#RcvDat			;ustaw adres pocztku bufora odb.
			DJNZ		ByteCnt,RDLoop		;sprawd licznik bajtw do odbioru
			SJMP		RDLast

RDLoop:	ACALL		RDAck					;pobierz dane, wylij potwierdzenie
			JB			Fault,RdatErr		;sprawd bdy transmisji
			MOV		@R0,A					;zapamitaj odebrany bajt
			INC		R0
			DJNZ		ByteCnt,RDLoop

RDLast:	ACALL		RcvByte				;pobierz ostatni bajt od slave
			JB			Fault,RdatErr		;sprawd bdy transmisj
			MOV		@R0,A					;zapamitaj odebrany bajt
			MOV		I2DAT,#80h			;wylij nack
			JNB		ATN,$					;czekaj na wysanie nack
			JNB		DRDY,RdatErr		;sprawd bdy transmisji

RDEX:		ACALL	SendStop					;wylij "I2C stop"
			RET

;obsuga bdu transmisji
RDatErr:	AJMP	Recover					;odtwrz stan SDA I SCL


;Wysanie danych do urzdzenia slave z adresem 10-bitowym
;adres slave w SlvAdr, sub-adres w SubAdr, liczba bajtw w ByteCnt,
;dane w XmtDat buffer.
SendSub:	CLR		NoAck					;kasowanie wsk. bdw
			CLR		Fault
			CLR		Retry
			MOV		StackSave,SP		;zapamitaj adresu stosu
			MOV		A,SlvAdr
			ACALL		SendAddr				;pobierz stan I2C i wylij adr.slave
			JB			NoAck,SSEX			;sprawd ack
			JB			Fault,SsubErr		;sprawd bdy transmisji

			MOV		A,SubAdr				;sub-adres
			ACALL		XmitByte				;wylij sub-adres
			JB			NoAck,SSEX			;sprawd ack
			JB			Fault,SsubErr		;sprawd bdy transmisji
			MOV		R0,#XmtDat			;ustaw adres bufora transmisji

SSLoop:	MOV		A,@R0					;pobierz dane dla slave
			INC		R0
			ACALL		XmitByte				;wylij dane do slave
			JB			NoAck,SSEX			;sprawd ack
			JB			Fault,SsubErr		;sprawd bdy transmisji
			DJNZ		ByteCnt,SSLoop

SSEX:		ACALL	SendStop					;wylij "I2C stop"
RET

;obsuga bdu transmisji
SSubErr:	AJMP		Recover				;odtworzenie stanu SDA I SCL


;Odbir danych od urzdzenia slave o adresie 10-bitowym
;adres slave w SlvAdr, sub-adres w SubAdr, liczba bajtw w
;ByteCnt. Dane zwracane s w buforze RcvDat.
RcvSub:	CLR		NoAck					;kasowanie wskanikw bdw
			CLR		Fault
			CLR		Retry
			MOV		StackSave,SP		;zapamitaj wskanik stosu
			MOV		A,SlvAdr				;pobierz adresu slave
			ACALL		SendAddr				;wylij adres slave
			JB			NoAck,RSEX			;sprawd ack
			JB			Fault,RsubErr		;sprawd bdy transmisji

			MOV		A,SubAdr				;pobierz sub-adres
			ACALL		XmitByte				;wylij sub-adres
			JB			NoAck,RSEX			;sprawd ack
			JB			Fault,RsubErr		;sprawd bdy transmisji

			ACALL		RepStart				;wylij powtrnie "I2C start"
			JB			Fault,RsubErr		;sprawd bdy transmisji
			MOV		A,SlvAdr				;pobierz adres slave
			SETB		ACC.0					;ustaw bit odczytu w adresie slave
			ACALL		SendAd2				;wylij adres slave
			JB			NoAck,RSEX			;sprawd ack
			JB			Fault,RsubErr		;sprawd bdy transmisji

			MOV		R0,#RcvDat			;ustaw adres bufora odbioru
			DJNZ		ByteCnt,RSLoop		;sprawd liczb odebranych bajtw
			SJMP		RSLast

RSLoop:	ACALL		RDAck					;pobierz dane i wylij ack
			JB			Fault,RsubErr		;sprawd bdy transmisji
			MOV		@R0,A					;zapamitaj dane
			INC		R0
			DJNZ		ByteCnt,RSLoop		;powtarzaj dla wszystkich bajtw

RSLast:	ACALL		RcvByte				;pobierz ostatni bajt od slave
			JB			Fault,RsubErr		;sprawd bdy transmisji
			MOV		@R0,A					;zapamitaj dane

			MOV		I2DAT,#80h			;wylij nack
			JNB		ATN,$					;czekaj na wysanie nack
			JNB		DRDY,RsubErr		;sprawd bdy transmisji

RSEX:		ACALL		SendStop				;wylij "I2C stop"
			RET

;obsuga bedu transmisji
RSubErr:	AJMP		Recover				;odtwrz stan SDA I SCL


;**************************************************************************
;                               Podprogramy
;**************************************************************************
;wysanie bajtu adresu, bajt podawany w akumulatorze
SendAddr:
			MOV		I2CFG,#BMRQ+BTIR+CTVAL		;danie dostpu do I2C w 
															;trybie master
			JNB		ATN,$					;oczekiwanie na przejcie kontroli
			JNB		Master,SAErr
SendAd2:	MOV		I2DAT,A				;wysanie bajtu, kasowanie DRDY
			MOV		I2CON,#BCARL+BCSTR+BCSTP	;kasowanie "I2C start", 
															;zwolnienie SCL
			ACALL		XmitAddr				;zakoczenie przesyania adresu
			RET

SAErr:	SETB		Fault					;sprawdzenie bdw transmisji
			RET


;Transmisja bajtu, bajt podawany w akumulatorze
;XmitByte : przesya 8 bitw
;XmitAddr : przesya 7 bitw (tylko do adresowania)
XmitAddr:	MOV	BitCnt,#8			;wysanie 7 bitw adresu
SJMP	XmBit2

XmitByte:
			MOV		BitCnt,#8			;wysanie 8 bitw danych
XmBit:	MOV		I2DAT,A				;wylij bit
XmBit2:	RL			A						;we nastpny bit
			JNB		ATN,$					;czekaj na wysanie bitu
			JNB		DRDY,XMErr			;sprawd gotowo
			DJNZ		BitCnt,XmBit		;powtarzaj dal wszystkich bitw
			MOV		I2CON,#BCDR+BCXA	;przecz w tryb odbioru
			JNB		ATN,$					;odbierz ack
			JNB		RDAT,XMBX			;byo ack?
			SETB		NoAck					;zwr "brak ack"
XMBX:		RET

XMErr:	SETB		Fault					;ustaw bit bdu transmisji
			RET


;Odbir bajtu, dane zwracane w akumulatorze
;RDAck   :odbiera bajt danych i wysya ack
;RcvByte :odbiera bajt danych
RDAck:	ACALL		RcvByte				;odbierz bajt danych
			MOV		I2DAT,#0				;wylij ack
			JNB		ATN,$					;czekaj na wysanie ack
			JNB		DRDY,RdErr			;sprawd bdy transmisji
			RET

RcvByte:	MOV		BitCnt,#8			;ustaw licznik bitw
			CLR		A						;inicjuj odebrany bajt
RBit:		ORL		A,I2DAT				;pobierz bit i skasuj ATN
			RL			A						;przesu dane w lewo 
			JNB		ATN,$					;czekaj na nastpny bit

			JNB		DRDY,RdErr			;testuj gotowo interfejsu
			DJNZ		BitCnt,Rbit			;powtarzaj odbir 7 bitw
			MOV		C,RDAT				;odbierz ostatni bit, nie kasuj ATN
			RLC		A			
			RET

RdErr:	SETB	Fault						;zwr komunikat o bdzie
RET


;wysanie "I2C stop"
SendStop:
			CLR		MASTRQ				;zwolnij magistral I2C
			MOV		I2CON,#BCDR+BXSTP	;wylij "I2C stop"
			JNB		ATN,$					;czekaj na ATN
			MOV		I2CON,#BCDR			;wycz gotowo na dane
			JNB		ATN,$					;czekaj na wysanie "I2C stop"
			MOV		I2CON,#BCARL+BCSTP+BCXA
			CLR		TIRUN					;zatrzymaj Timer 1
			RET


;Wysanie powtrzonego startu na I2C, adres w akumulatorze
RepStart:
			MOV		I2CON,#BCDR+BXSTR	;wylij powtrzony start
			JNB		ATN,$					;czekaj na ATN
			MOV		I2CON,#BCDR			;kasuj bit gotowoci na dane
			JNB		ATN,$					;czekaj na wysanie powtrzonego 
												;"I2C start"
			MOV		I2CON,#BCARL+BCSTR
			RET


;Procedura odtwarzania i zwalniania magistrali I2C
Recover:	ACALL		FixBus				;sprawd, czy magistrala funkcjonuje
			JC			BusReset				;jeli nie dziaa, sprbuj zmierzy
			SETB		Retry					;jeli magistrala ok, wr do programu
			CLR		Fault					;gwnego
			CLR		NoAck
			SETB		CLRTI
			SETB		TIRUN					;zacz Timer 1
			SETB		ETI					;zacz przerwania Timera 1
			RET


BusReset:
			CLR		MASTRQ				;zwolnij magistral
			MOV		I2CON,#0BCh			;kasuj flagi I2C
			SETB		TIRUN
			SJMP		$						;czekaj na przekroczenie limitu czasu
;operacji (po nim wewn.reset interfejsu)


;Procedura prbuje ponownie przej kontrol nad transmisj danych po
;bdzie transmisji; jeli operacja zakoczy si niepowodzeniem,
;ustawiana jest flaga CARRY
FixBus:	CLR		MastRQ				;wycz funkcje I2C
			SETB		C
			SETB		SCL					;upewnij si, e stan linii nie blokuje
			SETB		SDA					;magistrali I2C
			JNB		SCL,FixBusEx		;jeli SCL jest w stanie niskim,
												;magistrala nie moe by "naprawiona"
			JB			SDA,Rstop			;jeli SDA I SCL s w stanie wysokim,
												;wylij sygna "I2C stop"
			MOV		BitCnt,#9			;ustaw maksymaln liczb powtrze
ChekLoop:
			CLR		SCL					;zeruj lini zegara SCL
			ACALL		SDelay
			JB			SDA,Rstop			;sprawd, czy magistrala pracuje?
			SETB		SCL
			ACALL		SDelay
			DJNZ		BitCnt,ChekLoop	;powtarzaj wysyanie imp.SCL a SDA
												;osignie stan niski
			SJMP		FixBusEx				;skocz, jeli metoda nie powioda si

RStop:	CLR		SDA					;prbuj wysa "I2C stop" po ustawieniu
			ACALL		Sdelay				;SDA I SCL w stan wysoki
			SETB		SCL
			ACALL		SDelay
			SETB		SDA
			ACALL		SDelay
			JNB		SCL,FixBusEx		;sprawd, czy SDA i SCL s w stanie 
			JNB		SDA,FixBusEx		;wysokim; jeli tak, problem zosta
			CLR		C						;rozwizany - wr z flag CARRY
FixBusEx:
			RET								;rwn "0"


;krtkie opnienie (10 cykli maszynowych)
SDelay:	NOP
			NOP
			NOP
			NOP
			NOP
			NOP
			NOP
			NOP
			RET


;**************************************************************************
; Program gwny (przykad uycia procedur transmisji)
;**************************************************************************
Reset:	MOV		SP,#07h				;ustaw rozmiar stosu
			SETB		ETI					;zacz przerwania Timera 1
			SETB		EA						;zacz przyjmowanie przerwa
			MOV		XmtDat,#11			;ustaw dane do transmisji
			MOV		XmtDat+1,#22
			MOV		XmtDat+2,#44
			MOV		XmtDat+3,#88
			MOV		RcvDat,#0			;zeruj bufor odbioru
			MOV		RcvDat+1,#0
			MOV		RcvDat+2,#0
			MOV		RcvDat+3,#0

;odczyt i zapis urzdzenia slave o 8 bitowym adresie (przesyany 1 bajt
;danych)
MainLoop:
			MOV		SlvAdr,#48h			;ustaw adres slave (8-bit port I/O)
			MOV		ByteCnt,#1			;ustaw licznik bajtw
			ACALL		SendData				;wylij dane do slave
			JB			Retry,MainLoop

ML2:		MOV		ByteCnt,#1			;ustaw licznik bajtw
			ACALL		RcvData				;czytaj dane z urzdzenia slave
			JB			Retry,ML2

;zapis i odczyt urzdzenia slave o 10-bitowym adresie (przesyane 4 bajty
;danych)
SL1:		MOV		SlvAdr,#0A0h		;ustaw adres slave (RAM I2C)
			MOV		SubAdr,#0h			;ustaw sub-adres slave
			MOV		ByteCnt,#4			;ustaw liczb bajtw
			ACALL		SendSub
			JB			Retry,SL1

SL2:		MOV		ByteCnt,#4			;ustaw liczb bajtw
			ACALL		RcvSub
			JB			Retry,SL2

			INC		XmtDat
			INC		XmtDat+1
			INC		XmtDat+2
			INC		XmtDat+3
			SJMP		MainLoop

END

