;**************************************************************************
;assembler module - software SPI
;**************************************************************************
$include(reg_51.pdf)

ISP_asm segment code

RESET	bit	P3.3						;reset connection
MOSI	bit	P1.2						;mosi
MISO	bit	P1.3						;miso
SCK		bit	P1.4						;sck

public _wrser							;writing a byte through SPI
public rdser							;readin a byte from SPI
public _delayms							;delay 1 ms

public RESET							;public definitions
public MOSI
public MISO
public SCK


;**************************************************************************
rseg ISP_asm

;writing a byte using SPI; on input R7 contains the byte
;SCK should be 0 / function clears SCK after work
_wrser:		push	B						;B as bit counter
			mov	B,#8
			mov	A,R7						;move a byte for sending to accumulator
wrser_loop:
			rlc		A						;move less sign.bit to the C flag
			mov		MOSI,C					;move C flag to the MOSI line
			call	sck_pulse				;send SPI clock (SCK)
			djnz	B,wrser_loop			;repeat 8 times
			pop		B						;restore reg.B value
			ret

			
;function reads a byte from SPI and returns it in R7
;SCK should be 0 / function clears SCK after work
rdser:		push	B						;B as a bit counter
			mov		B,#8
rdser_loop:
			mov		C,MISO					;reading a bit (MISO contains a valid bit because of falling edge of SCK signal)
			rlc		A						;move a bit to the accumulator
			call	sck_pulse				;send SCK pulse
			djnz	B,rdser_loop			;repeat 8 times
			pop		B						;restore B value
			mov		R7,A					;return a byte in the R7
			ret


;sending SPI clock pulse (only one, SCK)
sck_pulse:
			setb	SCK						;rising edge
			mov		R7,#4					;small delay
			djnz	R7,$
			clr		SCK						;falling edge
			mov		R7,#4
			djnz	R7,$
			ret

			
;delay 1 milisecond
_delayms:
			push	ACC
			push	B
			mov	A,R7
			mov	B,#0
dloop:
			djnz	B,$						;~500us / 11.0592MHz
			djnz	B,$
			djnz	ACC,dloop
			
			pop	B
			pop	ACC
			ret


END
